<?php

/**
 * 
 * @example Usage Example (admin_item_related.php and admin_item_related.html)
 * 1) init ajax tree list and set it as ajax requests listener
 *	require($root_folder_path . "includes/ajax_list_tree.php");
 *	$list = new VA_Ajax_List_Tree($settings["admin_templates_dir"], "ajax_list_tree.html");
 *	$list->set_branches('categories', 'category_id', 'category_name', 'parent_category_id');
 *	$list->set_leaves('items', 'item_id', 'item_name', 'items_categories');
 *	$list->ajax_listen('products_ajax_tree', 'admin_item_related.php');
 * 	
 *	2) show root tree in your template 
 *	$list->parse_root_tree('products_ajax_tree', 'admin_item_related.php');
 * 
 *  3) add in template <script language="JavaScript" type= "text/javascript" src="../js/ajax_list_tree.js"></script>
 *  and {products_ajax_tree}
 */
class VA_Ajax_List_Tree {
	var $t;
	var $branches_table_name, $branches_id_field, $branches_name_field, $branches_recursion_field;
	var $branches_where, $branches_group, $branches_order;
	var $leaves_table_name,   $leaves_id_field,   $leaves_name_field,   $leaves_external_table_name;
	var $leaves_where, $leaves_group, $leaves_order;
	
	var $templates_path, $filename;
	
	var $action_object_id, $action_object_type, $leaf_action_type;

	function VA_Ajax_List_Tree($templates_path, $filename){
		$this->t = new VA_Template($templates_path);
		$this->t->set_file("ajax_list_tree", $filename);
		$this->templates_path = $templates_path; 
		$this->filename = $filename;		
	}
	function set_branches($table_name, $id_field, $name_field, $recursion_field, $where = null, $group = null, $order = null){
		$this->branches_table_name      = $table_name;
		$this->branches_id_field        = $id_field;
		$this->branches_name_field      = $name_field; 
		$this->branches_recursion_field = $recursion_field;
		
		if ($where && is_array($where)) {
			$where = implode(', ', $where);
		}
		if ($group && is_array($group)) {
			$group = implode(', ', $group);
		}
		if ($order && is_array($order)) {
			$order = implode(', ', $order);
		}
		$this->branches_where = $where;
		$this->branches_group = $group;
		$this->branches_order = $order;
	}
	function set_leaves($table_name, $id_field, $name_field, $external_table_name=null, $where = null, $group = null, $order = null ){
		$this->leaves_table_name          = $table_name;
		$this->leaves_id_field            = $id_field;
		$this->leaves_name_field          = $name_field; 
		$this->leaves_external_table_name = $external_table_name;
		
		if ($where && is_array($where)) {
			$where = implode(', ', $where);
		}
		if ($group && is_array($group)) {
			$group = implode(', ', $group);
		}
		if ($order && is_array($order)) {
			$order = implode(', ', $order);
		}
		$this->leaves_where = $where;
		$this->leaves_group = $group;
		$this->leaves_order = $order;
	}
	function set_actions($action_object_id, $action_object_type = 'select', $leaf_action_type = 'leaftostock') {
		$this->action_object_id   = $action_object_id;
		$this->action_object_type = $action_object_type;
		$this->leaf_action_type   = $leaf_action_type;		
	}
	function get_branches_sql ($root_id){
		global $db, $table_prefix;
		
		$sql  = " SELECT b." . $this->branches_id_field . ", b." .  $this->branches_name_field;
		$sql .= ", COUNT(bb." . $this->branches_id_field . ") AS branches_count ";

		$sql .= " FROM " . $table_prefix . $this->branches_table_name . " AS b ";
		$sql .= " LEFT JOIN " . $table_prefix . $this->branches_table_name . " AS bb ON bb." . $this->branches_recursion_field . " = b." . $this->branches_id_field;
		
		$sql .= " WHERE b." . $this->branches_recursion_field . "=" . $db->tosql($root_id,INTEGER, true, false);
		if ($this->branches_where) {
			$sql .= " AND " . $this->branches_where;
		}
				
		$sql .= " GROUP BY ";
		$sql .= " b." . $this->branches_id_field . ", b." .  $this->branches_name_field;
		if ($this->branches_group) {
			 $sql .= ", " . $this->branches_group;
		}
	
		$sql .= " ORDER BY ";
		if ($this->branches_order) {
			$sql .= $this->branches_order;
		} else {
			$sql .= " b." . $this->branches_name_field;
		}
		
		return $sql;
	}
	function get_leaves_sql ($root_id){
		global $db, $table_prefix;
		
		$sql  = " SELECT lv." . $this->leaves_id_field . ", lv." .  $this->leaves_name_field;
		$sql .= " FROM " . $table_prefix . $this->leaves_table_name . " AS lv ";
		if ($this->leaves_external_table_name) {
			$sql .= " LEFT JOIN " . $table_prefix . $this->leaves_external_table_name . " AS b ON b." . $this->leaves_id_field . " = lv." . $this->leaves_id_field;
			$sql .= " WHERE b." . $this->branches_id_field . "=" . $db->tosql($root_id,INTEGER, true, false);
		} else {
			$sql .= " WHERE lv." . $this->branches_id_field . "=" . $db->tosql($root_id,INTEGER, true, false);
		}
		
		if ($this->leaves_where) {
			$sql .= " AND " . $this->leaves_where;
		}
		
		$sql .= " GROUP BY ";
		$sql .= " lv." . $this->leaves_id_field . ", lv." .  $this->leaves_name_field;
		if ($this->leaves_group) {
			 $sql .= ", " . $this->leaves_group;
		}
		
		$sql .= " ORDER BY ";
		if ($this->leaves_order) {
			$sql .= $this->leaves_order;
		} else {
			$sql .= " lv." . $this->leaves_name_field;
		}
		
		return $sql;
	}
	function main($tree_name, $response_url, $root_id, $full = false, $marked = null){
		global $db, $table_prefix;
		global $db_type, $db_name, $db_host, $db_port, $db_user, $db_password, $db_persistent;
		
		$db2 = new VA_SQL();
		$db2->DBType      = $db_type;
		$db2->DBDatabase  = $db_name;
		$db2->DBHost      = $db_host;
		$db2->DBPort      = $db_port;
		$db2->DBUser      = $db_user;
		$db2->DBPassword  = $db_password;
		$db2->DBPersistent= $db_persistent;
		
		$this->t->set_var("tree_name", $tree_name);	
		$this->t->set_var("response_url", $response_url);	
		
		$db->query($this->get_branches_sql($root_id));
		while($db->next_record()) {			
			$id    = $db->f($this->branches_id_field);
			$name  = get_translation($db->f($this->branches_name_field));
			$branches_count = $db->f("branches_count");
			
			$sql = " SELECT COUNT(" . $this->leaves_id_field . ") ";
			if ($this->leaves_external_table_name) {
				$sql .= " FROM " . $table_prefix . $this->leaves_external_table_name;							
			} else if ($this->leaves_table_name) {
				$sql .= " FROM " . $table_prefix . $this->leaves_table_name;	
			}
			$sql .= " WHERE " . $this->branches_id_field . " = " . $db->tosql($id, INTEGER, false);
			$sql .= " GROUP BY " . $this->branches_id_field;		
			$db2->query($sql);
			if ($db2->next_record()) {
				$leaves_count = $db2->f(0);
			} else {
				$leaves_count = '';
			}
				
			$this->t->set_var("id", $id);
			$this->t->set_var("name", str_replace(array("\"", "'"), "&quot;", $name));	
			$this->t->set_var("branches_count", $branches_count);
			$this->t->set_var("leaves_count", $leaves_count);			
			$this->t->parse("branch", true);	
		}		

		$db->query($this->get_leaves_sql($root_id));		
		while($db->next_record()) {			
			$id    = $db->f($this->leaves_id_field);
			$name  = get_translation($db->f($this->leaves_name_field));
			$this->t->set_var("id", $id);
			$this->t->set_var("name", str_replace(array("\"", "'"), "&quot;", $name));	
			if($id != $marked) {
				$this->t->parse($this->leaf_action_type."_action", false);
				$this->t->set_var("current", '');
			} else {
				$this->t->set_var($this->leaf_action_type."_action", '');
				$this->t->parse("current", false);
			}
			$this->t->parse("leaf", true);	
		}	
		$this->t->set_var("action_object_id", $this->action_object_id);
		$this->t->set_var("action_object_type", $this->action_object_type);
		if ($full) {	
			$this->t->parse("treeinit_block");
			$this->t->parse($this->leaf_action_type."_block");	
		}
		$this->t->parse("tree", false);		
	}
	function parse_root_tree($tree_name, $response_url, $root_id = 0, $marked  = null){
		global $t;
		$this->main($tree_name, $response_url, $root_id, true, $marked );
		$t->set_var($tree_name, "<div id='$tree_name' class='tree_outer'>" . $this->t->get_var("tree") . "</div>");
		$t->sparse($tree_name . "_block", false);
	}
	function ajax_listen($tree_name, $response_url, $marked = null){
		if (!(isset($_SERVER["CONTENT_TYPE"]) && $_SERVER["CONTENT_TYPE"]=='application/ajax+html'))
			return 0;				
		$branch_id = get_param('branch_id');
		if ($branch_id) {
			$this->main($tree_name, $response_url, $branch_id, false, $marked);
			echo $this->t->get_var("tree");
		}
		exit;
	}
}
?>